@@ -24,6 +24,7 @@ this module can be modified.
 from __future__ import division, print_function, absolute_import
 
 import os
+import inspect
 import warnings
 import threading
 import contextlib
@@ -110,13 +111,24 @@ class settingsMeta(type):
             assert default_variable.value is not None
         return default_variable.value
 
-    @default.setter
-    def default(self, value):
-        raise AttributeError('Cannot assign settings.default')
-
     def _assign_default_internal(self, value):
         default_variable.value = value
 
+    def __setattr__(self, name, value):
+        if name == 'default':
+            raise AttributeError(
+                'Cannot assign to the property settings.default - '
+                'consider using settings.load_profile instead.'
+            )
+        elif not (isinstance(value, settingsProperty) or name.startswith('_')):
+            raise AttributeError(
+                'Cannot assign hypothesis.settings.%s=%r - the settings '
+                'class is immutable.  You can change the global default '
+                'settings with settings.load_profile, or use @settings(...) '
+                'to decorate your test instead.' % (name, value)
+            )
+        return type.__setattr__(self, name, value)
+
 
 class settings(
     settingsMeta('settings', (object,), {})  # type: ignore
@@ -212,6 +224,24 @@ class settings(
                 'settings objects can be called as a decorator with @given, '
                 'but test=%r' % (test,)
             )
+        if inspect.isclass(test):
+            from hypothesis.stateful import GenericStateMachine
+            if issubclass(test, GenericStateMachine):
+                attr_name = '_hypothesis_internal_settings_applied'
+                if getattr(test, attr_name, False):
+                    raise InvalidArgument(
+                        'Applying the @settings decorator twice would '
+                        'overwrite the first version; merge their arguments '
+                        'instead.'
+                    )
+                setattr(test, attr_name, True)
+                test.TestCase.settings = self
+                return test
+            else:
+                raise InvalidArgument(
+                    '@settings(...) can only be used as a decorator on '
+                    'functions, or on subclasses of GenericStateMachine.'
+                )
         if hasattr(test, '_hypothesis_internal_settings_applied'):
             note_deprecation(
                 '%s has already been decorated with a settings object, which '
@@ -317,10 +347,20 @@ class settings(
         bits = []
         for name, setting in all_settings.items():
             value = getattr(self, name)
+            # The only settings that are not shown are those that are
+            # deprecated and left at their default values.
             if value != setting.default or not setting.hide_repr:
                 bits.append('%s=%r' % (name, value))
         return 'settings(%s)' % ', '.join(sorted(bits))
 
+    def show_changed(self):
+        bits = []
+        for name, setting in all_settings.items():
+            value = getattr(self, name)
+            if value != setting.default:
+                bits.append('%s=%r' % (name, value))
+        return ', '.join(sorted(bits, key=len))
+
     def __enter__(self):
         note_deprecation(
             'Settings should be determined only by global state or with the '
@@ -594,7 +634,7 @@ class HealthCheck(Enum):
     def all(cls):
         # type: () -> List[HealthCheck]
         bad = (HealthCheck.exception_in_generation, HealthCheck.random_module)
-        return [h for h in list(cls) if h not in bad]
+        return [h for h in list(cls) if h not in bad]  # type: ignore
 
     exception_in_generation = 0
     """Deprecated and no longer does anything. It used to convert errors in
@@ -768,15 +808,13 @@ deadline and have not explicitly set a deadline yourself.
 
 settings._define_setting(
     'use_coverage',
-    default=True,
+    default=not_set,
+    deprecation_message="""
+use_coverage no longer does anything and can be removed from your settings.
+""",
     description="""
-Whether to use coverage information to improve Hypothesis's ability to find
-bugs.
-
-You should generally leave this turned on unless your code performs
-poorly when run under coverage. If you turn it off, please file a bug report
-or add a comment to an existing one about the problem that prompted you to do
-so.
+A flag to enable a feature that no longer exists. This setting is present
+only for backwards compatibility purposes.
 """
 )
 
@@ -792,15 +830,39 @@ class PrintSettings(Enum):
     """Make an educated guess as to whether it would be appropriate to print
     the blob.
 
-    The current rules are that this will print if both:
+    The current rules are that this will print if:
 
     1. The output from Hypothesis appears to be unsuitable for use with
-       :func:`~hypothesis.example`.
-    2. The output is not too long."""
+       :func:`~hypothesis.example`, and
+    2. The output is not too long, and
+    3. Verbosity is at least normal."""
 
     ALWAYS = 2
     """Always print a blob on failure."""
 
+    def __repr__(self):
+        return 'PrintSettings.%s' % (self.name,)
+
+
+def _validate_print_blob(value):
+    if isinstance(value, bool):
+        if value:
+            replacement = PrintSettings.ALWAYS
+        else:
+            replacement = PrintSettings.NEVER
+
+        note_deprecation(
+            'Setting print_blob=%r is deprecated and will become an error '
+            'in a future version of Hypothesis. Use print_blob=%r instead.' % (
+                value, replacement,
+            )
+        )
+        return replacement
+
+    # Values that aren't bool or PrintSettings will be turned into hard errors
+    # by the 'options' check.
+    return value
+
 
 settings._define_setting(
     'print_blob',
@@ -811,7 +873,9 @@ failures.
 
 See :ref:`the documentation on @reproduce_failure <reproduce_failure>` for
 more details of this behaviour.
-"""
+""",
+    validator=_validate_print_blob,
+    options=tuple(PrintSettings),
 )
 
 settings.lock_further_definitions()
@@ -825,7 +889,7 @@ def note_deprecation(message, s=None):
     verbosity = s.verbosity
     warning = HypothesisDeprecationWarning(message)
     if verbosity > Verbosity.quiet:
-        warnings.warn(warning, stacklevel=3)
+        warnings.warn(warning, stacklevel=2)
 
 
 settings.register_profile('default', settings())
