@@ -28,7 +28,7 @@ import collections
 import hypothesis.strategies as st
 from hypothesis.errors import InvalidArgument, ResolutionFailed
 from hypothesis.internal.compat import PY2, ForwardRef, abc, text_type, \
-    typing_root_type
+    binary_type, typing_root_type
 
 
 def type_sorting_key(t):
@@ -83,6 +83,8 @@ def from_typing_type(thing):
         if getattr(thing, '__tuple_use_ellipsis__', False) or \
                 len(elem_types) == 2 and elem_types[-1] is Ellipsis:
             return st.lists(st.from_type(elem_types[0])).map(tuple)
+        elif len(elem_types) == 1 and elem_types[0] == ():
+            return st.tuples()  # Empty tuple; see issue #1583
         return st.tuples(*map(st.from_type, elem_types))
     if isinstance(thing, typing.TypeVar):
         if getattr(thing, '__bound__', None) is not None:
@@ -131,7 +133,7 @@ _global_type_lookup = {
     fractions.Fraction: st.fractions(),
     decimal.Decimal: st.decimals(),
     text_type: st.text(),
-    bytes: st.binary(),
+    binary_type: st.binary(),
     datetime.datetime: st.datetimes(),
     datetime.date: st.dates(),
     datetime.time: st.times(),
